@@ -19,6 +19,7 @@
 */
 
 #include <glib.h>
+#include <string.h>
 
 #include <audacious/drct.h>
 #include <audacious/i18n.h>
@@ -40,20 +41,18 @@ enum
 {
   AOSD_TRIGGER_PB_START = 0,
   AOSD_TRIGGER_PB_TITLECHANGE = 1,
-  AOSD_TRIGGER_VOL_CHANGE = 2,
-  AOSD_TRIGGER_PB_PAUSEON = 3,
-  AOSD_TRIGGER_PB_PAUSEOFF = 4
+  AOSD_TRIGGER_PB_PAUSEON = 2,
+  AOSD_TRIGGER_PB_PAUSEOFF = 3
 };
 
 /* trigger codes array size */
-#define AOSD_TRIGGER_CODES_ARRAY_SIZE 5
+#define AOSD_TRIGGER_CODES_ARRAY_SIZE 4
 
 /* trigger codes array */
 gint aosd_trigger_codes[] =
 {
   AOSD_TRIGGER_PB_START,
   AOSD_TRIGGER_PB_TITLECHANGE,
-  AOSD_TRIGGER_VOL_CHANGE,
   AOSD_TRIGGER_PB_PAUSEON,
   AOSD_TRIGGER_PB_PAUSEOFF
 };
@@ -63,8 +62,6 @@ static void aosd_trigger_func_pb_start_onoff ( gboolean );
 static void aosd_trigger_func_pb_start_cb ( gpointer , gpointer );
 static void aosd_trigger_func_pb_titlechange_onoff ( gboolean );
 static void aosd_trigger_func_pb_titlechange_cb ( gpointer , gpointer );
-static void aosd_trigger_func_vol_change_onoff ( gboolean );
-static void aosd_trigger_func_vol_change_cb ( gpointer , gpointer );
 static void aosd_trigger_func_pb_pauseon_onoff ( gboolean );
 static void aosd_trigger_func_pb_pauseon_cb ( gpointer , gpointer );
 static void aosd_trigger_func_pb_pauseoff_onoff ( gboolean );
@@ -86,11 +83,6 @@ aosd_trigger_t aosd_triggers[] =
                                     aosd_trigger_func_pb_titlechange_onoff ,
                                     aosd_trigger_func_pb_titlechange_cb },
 
-  [AOSD_TRIGGER_VOL_CHANGE] = { N_("Volume Change") ,
-                                N_("Triggers OSD when volume is changed.") ,
-                                aosd_trigger_func_vol_change_onoff ,
-                                aosd_trigger_func_vol_change_cb },
-
   [AOSD_TRIGGER_PB_PAUSEON] = { N_("Pause On") ,
                                 N_("Triggers OSD when playback is paused.") ,
                                 aosd_trigger_func_pb_pauseon_onoff ,
@@ -140,7 +132,6 @@ aosd_trigger_start ( aosd_cfg_osd_trigger_t * cfg_trigger )
   }
   /* When called, this hook will display the text of the user pointer
      or the current playing song, if NULL */
-  hook_register("aosd toggle");
   hook_associate( "aosd toggle" , aosd_trigger_func_hook_cb , NULL );
   return;
 }
@@ -186,7 +177,7 @@ aosd_trigger_func_pb_start_onoff(gboolean turn_on)
 static void
 aosd_trigger_func_pb_start_cb(gpointer hook_data, gpointer user_data)
 {
-    gchar *title = aud_drct_pl_get_title(aud_drct_pl_get_pos());
+    char * title = aud_drct_get_title ();
 
     if (title != NULL)
     {
@@ -200,8 +191,8 @@ aosd_trigger_func_pb_start_cb(gpointer hook_data, gpointer user_data)
             g_free(utf8_title_markup);
         }
         g_free(utf8_title);
+        str_unref (title);
     }
-    return;
 }
 
 typedef struct
@@ -245,10 +236,8 @@ aosd_trigger_func_pb_titlechange_cb ( gpointer plentry_gp , gpointer prevs_gp )
     aosd_pb_titlechange_prevs_t *prevs = prevs_gp;
     gint playlist = aud_playlist_get_playing();
     gint pl_entry = aud_playlist_get_position(playlist);
-    const gchar * pl_entry_filename = aud_playlist_entry_get_filename (playlist,
-     pl_entry);
-    const gchar * pl_entry_title = aud_playlist_entry_get_title (playlist,
-     pl_entry, FALSE);
+    gchar * pl_entry_filename = aud_playlist_entry_get_filename (playlist, pl_entry);
+    gchar * pl_entry_title = aud_playlist_entry_get_title (playlist, pl_entry, FALSE);
 
     /* same filename but title changed, useful to detect http stream song changes */
 
@@ -291,66 +280,12 @@ aosd_trigger_func_pb_titlechange_cb ( gpointer plentry_gp , gpointer prevs_gp )
         g_free(prevs->filename);
       prevs->filename = g_strdup(pl_entry_filename);
     }
+
+    str_unref (pl_entry_filename);
+    str_unref (pl_entry_title);
   }
 }
 
-static void
-aosd_trigger_func_vol_change_onoff ( gboolean turn_on )
-{
-  if ( turn_on == TRUE )
-    hook_associate( "volume set" , aosd_trigger_func_vol_change_cb , NULL );
-  else
-    hook_dissociate( "volume set" , aosd_trigger_func_vol_change_cb );
-  return;
-}
-
-typedef struct
-{
-  gint h_vol[2];
-  gint sid;
-}
-aosd_vol_change_bucket_t;
-
-static gboolean
-aosd_trigger_func_vol_change_timeout ( gpointer bucket_gp )
-{
-  aosd_vol_change_bucket_t *bucket = bucket_gp;
-  gchar *utf8_title_markup = g_markup_printf_escaped(
-    "<span font_desc='%s'>Volume Change - L: %i , R: %i</span>" ,
-    global_config->osd->text.fonts_name[0] , bucket->h_vol[0] , bucket->h_vol[1] );
-  aosd_osd_display( utf8_title_markup , global_config->osd , FALSE );
-  g_free( utf8_title_markup );
-  bucket->sid = 0; /* reset source id value */
-  return FALSE;
-}
-
-static void
-aosd_trigger_func_vol_change_cb ( gpointer h_vol_gp , gpointer unused )
-{
-  gint *h_vol = h_vol_gp;
-  static aosd_vol_change_bucket_t bucket = { { 0 , 0 } , 0 };
-
-  bucket.h_vol[0] = h_vol[0];
-  bucket.h_vol[1] = h_vol[1];
-
-  /* in order to avoid repeated display of osd for each volume variation, use a
-     timer to prevent it from appearing more than once when multiple volume
-     changes are performed in a short time interval (500 msec) */
-  if ( bucket.sid == 0 )
-  {
-    /* first call in the time interval */
-    bucket.sid = g_timeout_add( 500 , aosd_trigger_func_vol_change_timeout , &bucket );
-  }
-  else
-  {
-    /* another call in the same interval, reset the interval */
-    g_source_remove( bucket.sid );
-    bucket.sid = g_timeout_add( 500 , aosd_trigger_func_vol_change_timeout , &bucket );
-  }
-  return;
-}
-
-
 static void
 aosd_trigger_func_pb_pauseon_onoff ( gboolean turn_on )
 {
@@ -387,7 +322,6 @@ aosd_trigger_func_pb_pauseoff_cb ( gpointer unused1 , gpointer unused2 )
 {
   gint active = aud_playlist_get_active();
   gint pos = aud_playlist_get_position(active);
-  const gchar * title;
   gchar *utf8_title, *utf8_title_markup;
   gint time_cur, time_tot;
   gint time_cur_m, time_cur_s, time_tot_m, time_tot_s;
@@ -399,14 +333,13 @@ aosd_trigger_func_pb_pauseoff_cb ( gpointer unused1 , gpointer unused2 )
   time_tot_s = time_tot % 60;
   time_tot_m = (time_tot - time_tot_s) / 60;
 
-  title = aud_playlist_entry_get_title (active, pos, FALSE);
-  utf8_title = aosd_trigger_utf8convert( title );
+  utf8_title = aud_playlist_entry_get_title (active, pos, FALSE);
   utf8_title_markup = g_markup_printf_escaped(
     "<span font_desc='%s'>%s (%i:%02i/%i:%02i)</span>" ,
     global_config->osd->text.fonts_name[0] , utf8_title , time_cur_m , time_cur_s , time_tot_m , time_tot_s );
   aosd_osd_display( utf8_title_markup , global_config->osd , FALSE );
   g_free( utf8_title_markup );
-  g_free( utf8_title );
+  str_unref (utf8_title);
   return;
 }
 
@@ -423,7 +356,7 @@ aosd_trigger_func_hook_cb ( gpointer markup_text , gpointer unused )
     aosd_osd_display( markup_text , global_config->osd , FALSE );
   } else {
     /* Display currently playing song */
-    aosd_trigger_func_pb_start_cb ( GINT_TO_POINTER(aud_drct_pl_get_pos()), NULL );
+    aosd_trigger_func_pb_start_cb (NULL, NULL);
   }
   return;
 }
