@@ -17,16 +17,12 @@
  * the use of this software.
  */
 
-extern float compressor_target, compressor_range;
-
 void compressor_config_load (void);
-void compressor_config_save (void);
 
-void compressor_init (void);
+int compressor_init (void);
 void compressor_cleanup (void);
 void compressor_start (int * channels, int * rate);
 void compressor_process (float * * data, int * samples);
 void compressor_flush (void);
 void compressor_finish (float * * data, int * samples);
-int compressor_decoder_to_output_time (int time);
-int compressor_output_to_decoder_time (int time);
+int compressor_adjust_delay (int delay);
