@@ -31,11 +31,8 @@
 #include <audacious/drct.h>
 #include <audacious/plugin.h>
 #include <audacious/i18n.h>
-#include <libaudgui/libaudgui.h>
-#include <libaudgui/libaudgui-gtk.h>
 
-static void init (void);
-static void about (void);
+static gboolean init (void);
 static void cleanup (void);
 void gnome_remote_init();
 void gnome_remote_uninit();
@@ -43,16 +40,19 @@ void gnome_remote_uninit();
 static gboolean loaded = FALSE;
 static DBusGProxy *media_player_keys_proxy = NULL;
 
-static GeneralPlugin audaciousgnomeshortcuts =
-{
-	.description = "Gnome Shortcuts",
-	.init = init,
-	.about = about,
-	.cleanup = cleanup
-};
+static const char about[] =
+ "Gnome Shortcut Plugin\n"
+ "Lets you control the player with Gnome's shortcuts.\n\n"
+ "Copyright (C) 2007-2008 Sascha Hlusiak <contact@saschahlusiak.de>";
 
-GeneralPlugin *gnomeshortcuts_gplist[] = { &audaciousgnomeshortcuts, NULL };
-SIMPLE_GENERAL_PLUGIN(gnomeshortcuts, gnomeshortcuts_gplist);
+AUD_GENERAL_PLUGIN
+(
+	.name = N_("Gnome Shortcuts"),
+	.domain = PACKAGE,
+	.about_text = about,
+	.init = init,
+	.cleanup = cleanup
+)
 
 #define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
 
@@ -91,7 +91,7 @@ static void
 on_media_player_key_pressed (DBusGProxy *proxy, const gchar *application, const gchar *key)
 {
 	if (strcmp ("Audacious", application) == 0) {
-		gint current_volume, old_volume;
+		gint current_volume /* , old_volume */ ;
 		static gint volume_static = 0;
 		gboolean play, mute;
 
@@ -100,7 +100,7 @@ on_media_player_key_pressed (DBusGProxy *proxy, const gchar *application, const
 
 		/* get current volume */
 		aud_drct_get_volume_main (&current_volume);
-		old_volume = current_volume;
+		/* old_volume = current_volume; */
 		if (current_volume)
 		{
 			/* volume is not mute */
@@ -298,21 +298,11 @@ void gnome_remote_init ()
 	}
 }
 
-static void about (void)
-{
-    static GtkWidget * dialog = NULL;
-
-    audgui_simple_message (& dialog, GTK_MESSAGE_INFO,
-     _("About Gnome Shortcut Plugin"),
-     _("Gnome Shortcut Plugin\n"
-     "Let's you control the player with Gnome's shortcuts.\n\n"
-     "Copyright (C) 2007-2008 Sascha Hlusiak <contact@saschahlusiak.de>\n\n"));
-}
-
-static void init (void)
+static gboolean init (void)
 {
 	gnome_remote_init();
 	loaded = TRUE;
+	return TRUE;
 }
 
 static void cleanup (void)
