@@ -22,15 +22,6 @@
 #define PLUGIN_SKINS_H
 
 #include <glib.h>
-#include <audacious/plugin.h>
-#include "skins_cfg.h"
-#include "ui_main.h"
-#include "ui_equalizer.h"
-#include "ui_playlist.h"
-#include "ui_skin.h"
-
-#define PACKAGE "audacious-plugins"
-#define PACKAGE_NAME "audacious-plugins"
 
 enum {
     SKINS_PATH_USER_SKIN_DIR,
@@ -39,11 +30,5 @@ enum {
 };
 
 extern gchar *skins_paths[];
-extern Interface skins_interface;
-
-gboolean skins_init(InterfaceCbs *cbs);
-gboolean skins_cleanup(void);
-void skins_about(void);
-void show_preferences_window(gboolean show);
 
 #endif
