@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2010 Audacious development team
+ *  Copyright (C) 2005-2011 Audacious development team
  *
  *  Based on the xmms_sndfile input plugin:
  *  Copyright (C) 2000, 2002 Erik de Castro Lopo
@@ -38,22 +38,20 @@
  *   - handle seeking/stopping while paused
  */
 
-#include "config.h"
-
-#include <glib.h>
 #include <math.h>
-#include <audacious/plugin.h>
-#include <audacious/i18n.h>
-#include <libaudgui/libaudgui.h>
-#include <libaudgui/libaudgui-gtk.h>
+#include <pthread.h>
+#include <stdlib.h>
 
 #include <sndfile.h>
 
-static GMutex * control_mutex;
-static GCond * control_cond;
-static gboolean pause_flag;
-static glong seek_value;
+#include <audacious/plugin.h>
+#include <audacious/i18n.h>
 
+#include "config.h"
+
+static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+static int seek_value;
+static bool_t stop_flag;
 
 /* Virtual file access wrappers for libsndfile
  */
@@ -96,77 +94,50 @@ static SF_VIRTUAL_IO sf_virtual_io =
     sf_tell
 };
 
-
-static SNDFILE *
-open_sndfile_from_uri(const gchar *filename, VFSFile **vfsfile, SF_INFO *sfinfo)
+static void copy_string (SNDFILE * sf, int sf_id, Tuple * tup, int tup_id)
 {
-    SNDFILE *snd_file = NULL;
-    *vfsfile = vfs_fopen(filename, "rb");
-
-    if (*vfsfile == NULL)
-        return NULL;
-
-    snd_file = sf_open_virtual (&sf_virtual_io, SFM_READ, sfinfo, *vfsfile);
-    if (snd_file == NULL)
-        vfs_fclose(*vfsfile);
-
-    return snd_file;
+    const char * str = sf_get_string (sf, sf_id);
+    if (str)
+        tuple_set_str (tup, tup_id, NULL, str);
 }
 
-static void
-close_sndfile(SNDFILE *snd_file, VFSFile *vfsfile)
+static void copy_int (SNDFILE * sf, int sf_id, Tuple * tup, int tup_id)
 {
-    if (snd_file != NULL)
-        sf_close(snd_file);
-    if (vfsfile != NULL)
-        vfs_fclose(vfsfile);
+    const char * str = sf_get_string (sf, sf_id);
+    if (str && atoi (str))
+        tuple_set_int (tup, tup_id, NULL, atoi (str));
 }
 
-
-/* Plugin initialization
- */
-static void plugin_init (void)
+static Tuple * get_song_tuple (const char * filename, VFSFile * file)
 {
-    control_mutex = g_mutex_new ();
-    control_cond = g_cond_new ();
-}
-
-static void plugin_cleanup (void)
-{
-    g_cond_free (control_cond);
-    g_mutex_free (control_mutex);
-}
-
-static Tuple * get_song_tuple (const gchar * filename)
-{
-    VFSFile *vfsfile = NULL;
     SNDFILE *sndfile;
     SF_INFO sfinfo;
-    gboolean lossy = FALSE;
-    gchar *codec, *format, *subformat;
+    const char *format, *subformat;
     Tuple * ti;
 
-    sndfile = open_sndfile_from_uri(filename, &vfsfile, &sfinfo);
+    sndfile = sf_open_virtual (& sf_virtual_io, SFM_READ, & sfinfo, file);
 
     if (sndfile == NULL)
         return NULL;
 
     ti = tuple_new_from_filename (filename);
 
-    if (sf_get_string(sndfile, SF_STR_TITLE) != NULL)
-        tuple_associate_string(ti, FIELD_TITLE, NULL, sf_get_string(sndfile, SF_STR_TITLE));
+    /* I have no idea version of sndfile ALBUM, GENRE, and TRACKNUMBER were
+     * added in. -jlindgren */
+    copy_string (sndfile, SF_STR_TITLE, ti, FIELD_TITLE);
+    copy_string (sndfile, SF_STR_ARTIST, ti, FIELD_ARTIST);
+/*    copy_string (sndfile, SF_STR_ALBUM, ti, FIELD_ALBUM); */
+    copy_string (sndfile, SF_STR_COMMENT, ti, FIELD_COMMENT);
+/*    copy_string (sndfile, SF_STR_GENRE, ti, FIELD_GENRE); */
+    copy_int (sndfile, SF_STR_DATE, ti, FIELD_YEAR);
+/*    copy_int (sndfile, SF_STR_TRACKNUMBER, ti, FIELD_TRACK_NUMBER); */
 
-    tuple_associate_string(ti, FIELD_ARTIST, NULL, sf_get_string(sndfile, SF_STR_ARTIST));
-    tuple_associate_string(ti, FIELD_COMMENT, NULL, sf_get_string(sndfile, SF_STR_COMMENT));
-    tuple_associate_string(ti, FIELD_DATE, NULL, sf_get_string(sndfile, SF_STR_DATE));
-    tuple_associate_string(ti, -1, "software", sf_get_string(sndfile, SF_STR_SOFTWARE));
-
-    close_sndfile (sndfile, vfsfile);
+    sf_close (sndfile);
 
     if (sfinfo.samplerate > 0)
     {
-        tuple_associate_int(ti, FIELD_LENGTH, NULL,
-        (gint) ceil (1000.0 * sfinfo.frames / sfinfo.samplerate));
+        tuple_set_int(ti, FIELD_LENGTH, NULL,
+        (int) ceil (1000.0 * sfinfo.frames / sfinfo.samplerate));
     }
 
     switch (sfinfo.format & SF_FORMAT_TYPEMASK)
@@ -261,55 +232,42 @@ static Tuple * get_song_tuple (const gchar * filename)
             break;
         case SF_FORMAT_ULAW:
             subformat = "U-Law";
-            lossy = TRUE;
             break;
         case SF_FORMAT_ALAW:
             subformat = "A-Law";
-            lossy = TRUE;
             break;
         case SF_FORMAT_IMA_ADPCM:
             subformat = "IMA ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_MS_ADPCM:
             subformat = "MS ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_GSM610:
             subformat = "GSM 6.10";
-            lossy = TRUE;
             break;
         case SF_FORMAT_VOX_ADPCM:
             subformat = "Oki Dialogic ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_G721_32:
             subformat = "32kbs G721 ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_G723_24:
             subformat = "24kbs G723 ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_G723_40:
             subformat = "40kbs G723 ADPCM";
-            lossy = TRUE;
             break;
         case SF_FORMAT_DWVW_12:
             subformat = "12 bit Delta Width Variable Word";
-            lossy = TRUE;
             break;
         case SF_FORMAT_DWVW_16:
             subformat = "16 bit Delta Width Variable Word";
-            lossy = TRUE;
             break;
         case SF_FORMAT_DWVW_24:
             subformat = "24 bit Delta Width Variable Word";
-            lossy = TRUE;
             break;
         case SF_FORMAT_DWVW_N:
             subformat = "N bit Delta Width Variable Word";
-            lossy = TRUE;
             break;
         case SF_FORMAT_DPCM_8:
             subformat = "8 bit differential PCM";
@@ -322,40 +280,18 @@ static Tuple * get_song_tuple (const gchar * filename)
     }
 
     if (subformat != NULL)
-        codec = g_strdup_printf("%s (%s)", format, subformat);
+    {
+        SPRINTF (codec, "%s (%s)", format, subformat);
+        tuple_set_format (ti, codec, sfinfo.channels, sfinfo.samplerate, 0);
+    }
     else
-        codec = g_strdup_printf("%s", format);
-
-    tuple_associate_string(ti, FIELD_CODEC, NULL, codec);
-    g_free(codec);
-
-    tuple_associate_string(ti, FIELD_QUALITY, NULL, lossy ? "lossy" : "lossless");
+        tuple_set_format (ti, format, sfinfo.channels, sfinfo.samplerate, 0);
 
     return ti;
 }
 
-static gint
-is_our_file (const gchar *filename)
-{
-    VFSFile *vfsfile = NULL;
-    SNDFILE *tmp_sndfile;
-    SF_INFO tmp_sfinfo;
-
-    /* Have to open the file to see if libsndfile can handle it. */
-    tmp_sndfile = open_sndfile_from_uri(filename, &vfsfile, &tmp_sfinfo);
-
-    if (!tmp_sndfile)
-        return FALSE;
-
-    /* It can so close file and return TRUE. */
-    close_sndfile (tmp_sndfile, vfsfile);
-    tmp_sndfile = NULL;
-
-    return TRUE;
-}
-
-static gboolean play_start (InputPlayback * playback, const gchar * filename,
- VFSFile * file, gint start_time, gint stop_time, gboolean pause)
+static bool_t play_start (InputPlayback * playback, const char * filename,
+ VFSFile * file, int start_time, int stop_time, bool_t pause)
 {
     if (file == NULL)
         return FALSE;
@@ -376,138 +312,91 @@ static gboolean play_start (InputPlayback * playback, const gchar * filename,
     /* Fix me!  Find out bitrate from libsndfile.  The old calculation was based
      * on the decoded data and therefore wrong for anything but floating-point
      * files. */
-    playback->set_params (playback, NULL, 0, 0, sfinfo.samplerate,
-     sfinfo.channels);
+    playback->set_params (playback, 0, sfinfo.samplerate, sfinfo.channels);
 
-    playback->playing = TRUE;
-    pause_flag = pause;
     seek_value = (start_time > 0) ? start_time : -1;
+    stop_flag = FALSE;
     playback->set_pb_ready(playback);
 
-    gint size = sfinfo.channels * (sfinfo.samplerate / 50);
-    gfloat * buffer = g_malloc (sizeof (gfloat) * size);
-    gboolean paused = FALSE, stopped = FALSE;
+    int size = sfinfo.channels * (sfinfo.samplerate / 50);
+    float * buffer = malloc (sizeof (float) * size);
 
     while (stop_time < 0 || playback->output->written_time () < stop_time)
     {
-        g_mutex_lock (control_mutex);
+        pthread_mutex_lock (& mutex);
 
-        if (! playback->playing)
+        if (stop_flag)
         {
-            stopped = TRUE;
-            g_mutex_unlock (control_mutex);
+            pthread_mutex_unlock (& mutex);
             break;
         }
 
         if (seek_value != -1)
         {
-            sf_seek (sndfile, (gint64) seek_value * sfinfo.samplerate / 1000,
+            sf_seek (sndfile, (int64_t) seek_value * sfinfo.samplerate / 1000,
              SEEK_SET);
             playback->output->flush (seek_value);
             seek_value = -1;
-            g_cond_signal (control_cond);
         }
 
-        if (pause_flag)
-        {
-            if (! paused)
-            {
-                playback->output->pause (TRUE);
-                paused = TRUE;
-                g_cond_signal (control_cond);
-            }
+        pthread_mutex_unlock (& mutex);
 
-            g_cond_wait (control_cond, control_mutex);
-            g_mutex_unlock (control_mutex);
-            continue;
-        }
-
-        if (paused)
-        {
-            playback->output->pause (FALSE);
-            paused = FALSE;
-            g_cond_signal (control_cond);
-        }
-
-        g_mutex_unlock (control_mutex);
-
-        gint samples = sf_read_float (sndfile, buffer, size);
+        int samples = sf_read_float (sndfile, buffer, size);
 
         if (! samples)
             break;
 
-        playback->output->write_audio (buffer, sizeof (gfloat) * samples);
+        playback->output->write_audio (buffer, sizeof (float) * samples);
     }
 
     sf_close (sndfile);
-    g_free (buffer);
+    free (buffer);
 
-    if (! stopped)
-    {
-        while (playback->output->buffer_playing ())
-            g_usleep (20000);
-    }
-
-    playback->output->close_audio();
-
-    g_mutex_lock (control_mutex);
-    playback->playing = FALSE;
-    g_cond_signal (control_cond); /* wake up any waiting request */
-    g_mutex_unlock (control_mutex);
+    pthread_mutex_lock (& mutex);
+    stop_flag = TRUE;
+    pthread_mutex_unlock (& mutex);
 
     return TRUE;
 }
 
-static void play_pause (InputPlayback * playback, gshort pause)
+static void play_pause (InputPlayback * p, bool_t pause)
 {
-    g_mutex_lock (control_mutex);
+    pthread_mutex_lock (& mutex);
 
-    if (! playback->playing)
-    {
-        g_mutex_unlock (control_mutex);
-        return;
-    }
+    if (! stop_flag)
+        p->output->pause (pause);
 
-    pause_flag = pause;
-    g_cond_signal (control_cond);
-    g_cond_wait (control_cond, control_mutex);
-    g_mutex_unlock (control_mutex);
+    pthread_mutex_unlock (& mutex);
 }
 
-static void play_stop (InputPlayback * playback)
+static void play_stop (InputPlayback * p)
 {
-    g_mutex_lock (control_mutex);
+    pthread_mutex_lock (& mutex);
 
-    if (! playback->playing)
+    if (! stop_flag)
     {
-        g_mutex_unlock (control_mutex);
-        return;
+        stop_flag = TRUE;
+        p->output->abort_write ();
     }
 
-    playback->playing = FALSE;
-    g_cond_signal (control_cond);
-    g_cond_wait (control_cond, control_mutex);
-    g_mutex_unlock (control_mutex);
+    pthread_mutex_unlock (& mutex);
 }
 
-static void file_mseek (InputPlayback * playback, gulong time)
+static void file_mseek (InputPlayback * p, int time)
 {
-    g_mutex_lock (control_mutex);
+    pthread_mutex_lock (& mutex);
 
-    if (! playback->playing)
+    if (! stop_flag)
     {
-        g_mutex_unlock (control_mutex);
-        return;
+        seek_value = time;
+        p->output->abort_write();
     }
 
-    seek_value = time;
-    g_cond_signal (control_cond);
-    g_cond_wait (control_cond, control_mutex);
-    g_mutex_unlock (control_mutex);
+    pthread_mutex_unlock (& mutex);
 }
 
-static gint
-is_our_file_from_vfs(const gchar *filename, VFSFile *fin)
+static int
+is_our_file_from_vfs(const char *filename, VFSFile *fin)
 {
     SNDFILE *tmp_sndfile;
     SF_INFO tmp_sfinfo;
@@ -525,49 +414,34 @@ is_our_file_from_vfs(const gchar *filename, VFSFile *fin)
     return TRUE;
 }
 
+const char plugin_about[] =
+ "Based on the xmms_sndfile plugin:\n"
+ "Copyright (C) 2000, 2002 Erik de Castro Lopo\n\n"
+ "Adapted for Audacious by Tony Vroon <chainsaw@gentoo.org>\n\n"
+ "This program is free software; you can redistribute it and/or "
+ "modify it under the terms of the GNU General Public License "
+ "as published by the Free Software Foundation; either version 2 "
+ "of the License, or (at your option) any later version.\n\n"
+ "This program is distributed in the hope that it will be useful, "
+ "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+ "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+ "GNU General Public License for more details.\n\n"
+ "You should have received a copy of the GNU General Public License "
+ "along with this program; if not, write to the Free Software "
+ "Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.";
 
-static void plugin_about (void)
-{
-    static GtkWidget * aboutbox = NULL;
+static const char *sndfile_fmts[] = { "aiff", "au", "raw", "wav", NULL };
 
-    audgui_simple_message (& aboutbox, GTK_MESSAGE_INFO,
-     _("About sndfile plugin"),
-     _("Adapted for Audacious usage by Tony Vroon <chainsaw@gentoo.org>\n"
-     "from the xmms_sndfile plugin which is:\n"
-     "Copyright (C) 2000, 2002 Erik de Castro Lopo\n\n"
-     "This program is free software ; you can redistribute it and/or modify \n"
-     "it under the terms of the GNU General Public License as published by \n"
-     "the Free Software Foundation ; either version 2 of the License, or \n"
-     "(at your option) any later version. \n \n"
-     "This program is distributed in the hope that it will be useful, \n"
-     "but WITHOUT ANY WARRANTY ; without even the implied warranty of \n"
-     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  \n"
-     "See the GNU General Public License for more details. \n\n"
-     "You should have received a copy of the GNU General Public \n"
-     "License along with this program ; if not, write to \n"
-     "the Free Software Foundation, Inc., \n"
-     "51 Franklin Street, Fifth Floor, \n"
-     "Boston, MA  02110-1301  USA"));
-}
-
-
-static const gchar *sndfile_fmts[] = { "aiff", "au", "raw", "wav", NULL };
-
-static InputPlugin sndfile_ip = {
-    .description = "sndfile plugin",
-    .init = plugin_init,
-    .about = plugin_about,
-    .is_our_file = is_our_file,
+AUD_INPUT_PLUGIN
+(
+    .name = N_("Sndfile Plugin"),
+    .domain = PACKAGE,
+    .about_text = plugin_about,
     .play = play_start,
     .stop = play_stop,
     .pause = play_pause,
-    .cleanup = plugin_cleanup,
-    .get_song_tuple = get_song_tuple,
+    .probe_for_tuple = get_song_tuple,
     .is_our_file_from_vfs = is_our_file_from_vfs,
-    .vfs_extensions = sndfile_fmts,
+    .extensions = sndfile_fmts,
     .mseek = file_mseek,
-};
-
-static InputPlugin *sndfile_iplist[] = { &sndfile_ip, NULL };
-
-SIMPLE_INPUT_PLUGIN(sndfile, sndfile_iplist)
+)
