@@ -1,5 +1,5 @@
 /*
- *  Copyright 2000,2001 Haavard Kvaalen <havardk@sol.no>
+ *  Copyright 2000, 2001 Håvard Kvålen <havardk@sol.no>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,11 +21,10 @@
 #include <audacious/i18n.h>
 #include <audacious/misc.h>
 #include <audacious/plugin.h>
-#include <libaudgui/libaudgui.h>
-#include <libaudgui/libaudgui-gtk.h>
 
 #include <glib.h>
 #include <stdlib.h>
+#include <string.h>
 #include <math.h>
 
 #define MIN_FREQ        10
@@ -38,27 +37,9 @@
 #define PI              3.14159265358979323846
 #endif
 
-static InputPlugin tone_ip;
+static gboolean stop_flag = FALSE;
 
-static void tone_init(void)
-{
-    aud_uri_set_plugin("tone://", &tone_ip);
-}
-
-static void tone_about (void)
-{
-    static GtkWidget * aboutbox = NULL;
-
-    audgui_simple_message (& aboutbox, GTK_MESSAGE_INFO,
-     _("About Tone Generator"),
-     /* I18N: UTF-8 Translation: "Haavard Kvaalen" -> "H\303\245vard Kv\303\245len" */
-     _("Sinus tone generator by Haavard Kvaalen <havardk@xmms.org>\n"
-     "Modified by Daniel J. Peng <danielpeng@bigfoot.com>\n\n"
-     "To use it, add a URL: tone://frequency1;frequency2;frequency3;...\n"
-     "e.g. tone://2000;2005 to play a 2000Hz tone and a 2005Hz tone"));
-}
-
-static gint tone_is_our_file(const gchar * filename)
+static gboolean tone_is_our_fd(const gchar *filename, VFSFile *fd)
 {
     if (!strncmp(filename, "tone://", 7))
         return TRUE;
@@ -114,12 +95,13 @@ static gchar *tone_title(const gchar * filename)
     return title;
 }
 
-static void tone_play(InputPlayback * playback)
+static gboolean tone_play(InputPlayback *playback, const gchar *filename,
+    VFSFile *file, gint start_time, gint stop_time, gboolean pause)
 {
     GArray *frequencies;
-    gchar *name, *filename = playback->filename;
     gfloat data[BUF_SAMPLES];
     gsize i;
+    gboolean error = FALSE;
     struct
     {
         gdouble wd;
@@ -128,17 +110,18 @@ static void tone_play(InputPlayback * playback)
 
     frequencies = tone_filename_parse(filename);
     if (frequencies == NULL)
-        return;
+        return FALSE;
 
     if (playback->output->open_audio(FMT_FLOAT, OUTPUT_FREQ, 1) == 0)
     {
-        playback->error = TRUE;
+        error = TRUE;
         goto error_exit;
     }
 
-    name = tone_title(filename);
-    playback->set_params(playback, name, -1, 16 * OUTPUT_FREQ, OUTPUT_FREQ, 1);
-    g_free(name);
+    if (pause)
+        playback->output->pause(TRUE);
+
+    playback->set_params(playback, 16 * OUTPUT_FREQ, OUTPUT_FREQ, 1);
 
     tone = g_malloc(frequencies->len * sizeof(*tone));
     for (i = 0; i < frequencies->len; i++)
@@ -149,11 +132,10 @@ static void tone_play(InputPlayback * playback)
         tone[i].t = 0;
     }
 
-    playback->playing = TRUE;
-    playback->error = FALSE;
+    stop_flag = FALSE;
     playback->set_pb_ready(playback);
 
-    while (playback->playing)
+    while (!stop_flag)
     {
         for (i = 0; i < BUF_SAMPLES; i++)
         {
@@ -171,28 +153,32 @@ static void tone_play(InputPlayback * playback)
             data[i] = (sum_sines * 0.999 / (gdouble) frequencies->len);
         }
 
-        playback->pass_audio(playback, FMT_FLOAT, 1, BUF_BYTES, data, &playback->playing);
+        if (!stop_flag)
+            playback->output->write_audio(data, BUF_BYTES);
     }
 
 error_exit:
     g_array_free(frequencies, TRUE);
     g_free(tone);
 
-    playback->output->close_audio();
-    playback->playing = FALSE;
+    stop_flag = TRUE;
+
+    return !error;
 }
 
 static void tone_stop(InputPlayback * playback)
 {
-    playback->playing = FALSE;
+    stop_flag = TRUE;
+    playback->output->abort_write();
 }
 
-static void tone_pause(InputPlayback * playback, gshort paused)
+static void tone_pause(InputPlayback * playback, gboolean pause)
 {
-    playback->output->pause(paused);
+    if (!stop_flag)
+        playback->output->pause(pause);
 }
 
-static Tuple *tone_get_song_tuple(const gchar * filename)
+static Tuple *tone_probe_for_tuple(const gchar *filename, VFSFile *fd)
 {
     Tuple *tuple = tuple_new_from_filename(filename);
     gchar *tmp;
@@ -202,24 +188,30 @@ static Tuple *tone_get_song_tuple(const gchar * filename)
 
     if ((tmp = tone_title(filename)) != NULL)
     {
-        tuple_associate_string(tuple, FIELD_TITLE, NULL, tmp);
+        tuple_set_str(tuple, FIELD_TITLE, NULL, tmp);
         g_free(tmp);
     }
 
     return tuple;
 }
 
-static InputPlugin tone_ip = {
-    .description = "Tone Generator",
-    .init = tone_init,
-    .about = tone_about,
-    .is_our_file = tone_is_our_file,
-    .play_file = tone_play,
+static const char tone_about[] =
+ "Sine tone generator by Håvard Kvålen <havardk@xmms.org>\n"
+ "Modified by Daniel J. Peng <danielpeng@bigfoot.com>\n\n"
+ "To use it, add a URL: tone://frequency1;frequency2;frequency3;...\n"
+ "e.g. tone://2000;2005 to play a 2000 Hz tone and a 2005 Hz tone";
+
+static const gchar * const schemes[] = {"tone", NULL};
+
+AUD_INPUT_PLUGIN
+(
+    .name = N_("Tone Generator"),
+    .domain = PACKAGE,
+    .about_text = tone_about,
+    .schemes = schemes,
+    .is_our_file_from_vfs = tone_is_our_fd,
+    .play = tone_play,
     .stop = tone_stop,
     .pause = tone_pause,
-    .get_song_tuple = tone_get_song_tuple
-};
-
-static InputPlugin *tonegen_iplist[] = { &tone_ip, NULL };
-
-DECLARE_PLUGIN(tonegen, NULL, NULL, tonegen_iplist, NULL, NULL, NULL, NULL, NULL);
+    .probe_for_tuple = tone_probe_for_tuple
+)
